/*	$LAAS: $ */

/* 
 * Copyright (c) 2000-2004 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef H_LIBT3D
#define H_LIBT3D

/* T3D error codes */
typedef enum T3DError {
#define T3D_MAGIC 0x3580 
   T3D_EOK =			T3D_MAGIC | 0,
   T3D_EBADTYPE =		T3D_MAGIC | 1,
   T3D_ETYPESDIFFER =		T3D_MAGIC | 2,
   T3D_ENOTIMPLEMENTED =	T3D_MAGIC | 15
} T3DError;

/* T3D types */
typedef enum T3DType {
  T3D_MATRIX,
  T3D_EULER,
  T3D_VECTOR
} T3DType;

/* T3D flags */
typedef enum T3DFlags {
   T3D_MINIMAL_SIZE =		1<<0,
   T3D_ALLOW_CONVERSION =	1<<1
} T3DFlags;


/* --- T3D data structures ------------------------------------------- */

/* Matrix */
typedef struct T3DMatrix {
  short type; 			/* T3D_MATRIX */
  short flags;
  int err;
  double matrix[16];		/* homogeneous matrix */
  double v[21];			/* covariance matrix on _Euler_ angles */
} T3DMatrix;


/* Euler */
typedef struct T3DEuler {
  short type;			/* T3D_EULER */
  short flags;
  int err;
  double euler[6];		/* alpha, beta, gamma, Tx, Ty, Tz */
  double v[21];
} T3DEuler;

/* Vector with variance  */
typedef struct T3DVector {
  short type; 			/* T3D_VECTOR */
  short flags; 
  int err;
  double vector[6];		/* rx,ry,rz, tx,ty,tz */
  double v[21];
} T3DVector;

/* Any representation  */
typedef struct T3DAny {
  short type; 			/* anything */
  short flags; 
  int err;
  double pad[37];
} T3DAny;


/* 
 * This union is defined so that T3D can manipulate arbitrary
 * representation with the same functions.
 */
typedef union T3D {
  short type; 			/* must not be changed */
  T3DAny any;
  T3DMatrix matrix;
  T3DEuler euler;
  T3DVector vector;
} T3D;


/* initializers for static structs */
extern const T3DEuler t3dEulerIdentity;
extern const T3DVector t3dVectorIdentity;
extern const T3DMatrix t3dMatrixIdentity;


/* --- API ----------------------------------------------------------- */

#define t3dOk(t)	((t) && (t)->any.err == T3D_EOK)
#define t3dSetOk(t)	((t)->any.err = T3D_EOK)


T3D *	t3dNew(T3DType type, int flags);
T3D *	t3dDup(T3D *t3d);
void	t3dDelete(T3D *t3d);

T3D *	t3dCopyIn(T3D *cpy, T3D *src);
T3D *	t3dInvert(T3D *t3d);
T3D *	t3dConvertTo(T3DType typeTo, T3D *t3d);

T3D *	t3dComp(T3D *a, T3D *b);
T3D *	t3dCompIn(T3D *c, T3D *a, T3D *b);

#endif /* H_LIBT3D */
