/*	$LAAS: $ */

/* 
 * Copyright (c) 2000-2004 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifdef VXWORKS
#include <vxWorks.h>
#endif

#if defined(__RTAI__) && defined(__KERNEL__)
# include <linux/kernel.h>
# include <linux/slab.h>
#else
# include <stdio.h>
# include <stdlib.h>
#endif

#include "libt3d.h"
#include "t3d.h"


const T3DMatrix t3dMatrixIdentity = {
   T3D_MATRIX, 0, T3D_EOK,
   { 1.0, 0.0, 0.0,     0.0,
     0.0, 1.0, 0.0,     0.0,
     0.0, 0.0, 1.0,     0.0,
     0.0, 0.0, 0.0,     1.0 },

   { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
          0.0, 0.0, 0.0, 0.0, 0.0,
               0.0, 0.0, 0.0, 0.0,
                    0.0, 0.0, 0.0,
                         0.0, 0.0,
                              0.0 }};


const T3DEuler t3dEulerIdentity = {
   T3D_EULER, 0, T3D_EOK,
   { 0.0, 0.0, 0.0,     0.0, 0.0, 0.0 },

   { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
          0.0, 0.0, 0.0, 0.0, 0.0,
               0.0, 0.0, 0.0, 0.0,
                    0.0, 0.0, 0.0,
                         0.0, 0.0,
                              0.0 }};


const T3DVector t3dVectorIdentity = {
   T3D_VECTOR, 0, T3D_EOK,
   { 0.0, 0.0, 0.0,     0.0, 0.0, 0.0 },

   { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
          0.0, 0.0, 0.0, 0.0, 0.0,
             0.0, 0.0, 0.0, 0.0,
                    0.0, 0.0, 0.0,
                         0.0, 0.0,
                              0.0 }};

/* for internal use */
const double t3dmid3[9] = {
   1.0, 0.0, 0.0,
   0.0, 1.0, 0.0,
   0.0, 0.0, 1.0
};

#if defined(__RTAI__) && defined(__KERNEL__)
# define malloc(x)	kmalloc(x, GFP_KERNEL)
# define free(x)	kfree(x)
#endif


/*
 * t3dNew ---------------------------------------------------------------
 *
 * Malloc a new structure for a tr3d and initialize it with the type
 * "type". The size of the new structure is compatible with any type of
 * representation except if the T3D_MINIMAL_SIZE flag is present.
 */

T3D *
t3dNew(T3DType type, int flags)
{
   T3D *new;
   
   if (flags & T3D_MINIMAL_SIZE) {
      switch (type) {
	 case T3D_MATRIX:	new = malloc(sizeof(T3DMatrix)); break;
	 case T3D_EULER:	new = malloc(sizeof(T3DEuler));  break;
	 case T3D_VECTOR:	new = malloc(sizeof(T3DVector)); break;
	 default:		new = malloc(sizeof(T3D));       break;
      }
   } else new = malloc(sizeof(T3D));

   if (new == NULL) return NULL;
   new->any.flags = 0;
   new->any.err = T3D_EOK;

   switch(type) {
      case T3D_MATRIX:
	 t3dCopyIn(new, (T3D *)&t3dMatrixIdentity);	break;
      case T3D_EULER:
	 t3dCopyIn(new, (T3D *)&t3dEulerIdentity);	break;
      case T3D_VECTOR:
	 t3dCopyIn(new, (T3D *)&t3dVectorIdentity);	break;
     
      default:
	 new->type = type;
	 new->any.err = T3D_EBADTYPE;
	 t3dWarn();
	 break;
   }
   
   new->any.flags |= flags;
   return new;
}


/*
 * t3dDup ---------------------------------------------------------------
 *
 * Malloc a new structure for a tr3d and initialize it with t3d
 */

T3D *
t3dDup(T3D *t3d)
{
   if (!t3dOk(t3d)) { t3dWarn(); return NULL; }

   return t3dCopyIn(t3dNew(t3d->type, t3d->any.flags), t3d);
}


/*
 * t3dDelete ------------------------------------------------------------
 *
 * Free an malloc'ed T3D.
 */

void
t3dDelete(T3D *t3d)
{
   if (!t3dOk(t3d)) { t3dWarn(); return; }

   free(t3d);
}


/*
 * t3dWarn --------------------------------------------------------------
 *
 * Called when errno is set or when an errno != T3D_EOK is detected
 */

void
t3dWarn()
{
#if defined(__RTAI__) && defined(__KERNEL__)
   printk("t3d: error while manipulating a T3D\n");
#else
   fputs("t3d: error while manipulating a T3D\n", stderr);
#endif
}
